from flask import request, jsonify, g
from sqlalchemy import select
from flask_restx import Resource, Namespace, reqparse
from sqlalchemy.orm import Session

from app.database import engine
from app.modle.users import User
from app.utils.jwt_util import generate_jwt
from app.defines import StatesCode, Module, OperationType
from app.utils.save_log import save_log

ns = Namespace('login', description='登入')

login = reqparse.RequestParser(bundle_errors=True)
login.add_argument(name='username', type=str, required=True, location='form', help='用户名')
login.add_argument(name='password', type=str, required=True, location='form', help='密码')


@ns.route('')
class LoginApi(Resource):
    @ns.doc(id='login', description='登入')
    @ns.expect(login)
    def post(self):
        """登入"""
        username = request.form.get('username')
        password = request.form.get('password')

        with Session(engine) as session:
            stmt = select(User).where(User.user_name == username)
            result = session.execute(stmt).scalars().first()
            # statement = session.query(User).filter_by(user_name=username).first()

        if result is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message='用户不存在')

        # 是否为禁用状态
        if result.account_status:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message='用户已禁用')

        # 验证用户名密码
        if (result.user_name == username) and (result.check_password(password) == True):
            # 生成token
            token = generate_jwt(result.id, result.name)

            g.user_name = result.name  # 存错用户名，用于日志写入
            save_log(request, Module.LOGIN, OperationType.LOGIN, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message="登录成功", date=token)
        else:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="用户名或密码错误")
